<?php


if (file_exists(cpg_plugin_dir_path() . '/openspout/autoload.php')){
require_once(cpg_plugin_dir_path() . '/openspout/autoload.php');
}

use OpenSpout\Reader\Common\Creator\ReaderEntityFactory;
use OpenSpout\Writer\Common\Creator\WriterEntityFactory;
use OpenSpout\Writer\Common\Creator\Style\StyleBuilder;
use OpenSpout\Common\Entity\Style\CellAlignment;
use OpenSpout\Common\Entity\Style\Color;

#use OpenSpout\Reader\XLSX\Reader;


class OpenSpoutExcel {

	var $runtime = 0;
	var $memory = '';

	function __construct() {

	}

	function ReadExcel($filename, $sheetnames='', $opts=array()) {
		$start = microtime(true);

		if (file_exists($filename)) {
			$reader = ReaderEntityFactory::createReaderFromFile($filename);

			$fileparts = explode(".", $filename);
			$ext = array_pop($fileparts);
			switch (strtolower($ext)) {
				case 'csv':
					$reader->setFieldDelimiter('|');
					$reader->setFieldEnclosure('@');
					break;
				case 'xlsx':
					break;
				case 'ods':
					break;
				default:

			}

			if (empty($sheetnames) || $sheetnames=="all") {
				$array = $this->ReadAllSheets($reader, $filename);
				$end = microtime(true);
				$this->runtime = 'Run time = ' . number_format(($end - $start), 2) . ' seconds';
				$this->memory =  '#'.floor((memory_get_peak_usage())/1024/1024)."MB peak memory used "."\n";

				return $array;
			} else {
				$array = $this->ReadSheetByName($reader, $filename, $sheetnames);
				$end = microtime(true);
				$this->runtime = 'Run time = ' . number_format(($end - $start), 2) . ' seconds';
				$this->memory =  '#'.floor((memory_get_peak_usage())/1024/1024)."MB peak memory used "."\n";

				return $array;
			}
		} else {
			$errors = new WP_Error();
			$errors->add('missing_file', 'The spreadsheet filename does not exist');
			return $errors;
		}
		
	} # end of ReadExcel

	function ReadAllSheets( $reader, $filename ) {
		$array = array();
		$reader->open($filename);
		foreach ($reader->getSheetIterator() as $sheet) {
			foreach ($sheet->getRowIterator() as $row) {
				// do stuff with the row
				# $cells = $row->getCells();
				$array[$sheet->getName()][] = $row->toArray();
			}
		}
		$reader->close();
		return $array;
	} # end of ReadAllSheets

	function ReadSheetByName( $reader, $filename, $sheetname ) {
		$array = array();
		if (!is_array($sheetname)) {
			$sheetname = array( $sheetname );
		}
		$reader->open($filename);
		foreach ($reader->getSheetIterator() as $sheet) {
			#if ($sheet->getName() === $sheetname) {
			if (in_array($sheet->getName(), $sheetname)) {
				foreach ($sheet->getRowIterator() as $row) {
					// do stuff with the row
					# $cells = $row->getCells();
					$array[$sheet->getName()][] = $row->toArray();
				}
			}
		}
		$reader->close();
		return $array;
	} # end of ReadAllSheets

	function ImportSheetByName( $filename, $sheetname ) {
		$array = array();
		if (!is_array($sheetname)) {
			$sheetname = array( $sheetname );
		}
		$reader = ReaderEntityFactory::createReaderFromFile($filename);

		$reader->open($filename);
		$result = array();
		foreach ($reader->getSheetIterator() as $sheet) {
			#if ($sheet->getName() === $sheetname) {
			$headers = array();

			$sheet_name = $sheet->getName();

			if (in_array($sheet->getName(), $sheetname)) {
				$loop = 0;
				foreach ($sheet->getRowIterator() as $row) {
					// do stuff with the row
					if (empty($headers)) {
						$headers = array_values($row->toArray());
						continue;
					}
					$loop++;
					$result[$sheet_name]['count'] = $loop;

					$array = array($row->toArray());

					processImport($sheet_name, $headers, $array);
				}
			}
		}
		$this->memory =  '#'.floor((memory_get_peak_usage())/1024/1024)."MB peak memory used "."\n";

		$reader->close();
		return $result;
 	}

	function WriteExcel( $array, $filename, $sheetname='') {

		$writer = WriterEntityFactory::createWriterFromFile($filename);
		$writer->setShouldUseInlineStrings(true); // default (and recommended) value
		#$writer->setShouldUseInlineStrings(false); // will use shared strings
		#$writer->setShouldAddBOM(false);

/** Create a style with the StyleBuilder */
$style = (new StyleBuilder())
           ->setFontBold()
           ->setFontSize(15)
           ->setFontColor(Color::BLUE)
           ->setShouldWrapText()
           ->setCellAlignment(CellAlignment::RIGHT)
           ->setBackgroundColor(Color::YELLOW)
           ->build();

		$writer->openToFile($filename);

		foreach ($array as $entry) {

			$entry = $this->remove_row_cols($entry);

			$row = WriterEntityFactory::createRowFromArray($entry);
			$writer->addRow($row);

		}

		$writer->close();
	}


	function WriteCSVFile( $array, $filename, $convertXL = true ) {

		if (stripos($filename, "/")!==false) {
			// filename has the path
			$parts = explode("/", $filename) ;

			$filename = array_pop($parts);
			$path = implode("/", $parts) . "/";
		} else {
			$uploads = wp_upload_dir();
			$path = $uploads['path'] . "/";
		}

		// ensure we are saving a csv file
		$file = str_replace('.php', '.csv', $path . $filename);

		$rowcount = count($array);
		echo "<br>Count=" . $rowcount;
		$chunksize = 1000;
		$pointer = 0;
		$append = false;

		while ($pointer<=$rowcount) {
			$chunk = array_slice($array, $pointer, $chunksize);
	
			$this->WriteFileSnippet($file, $chunk, $append) ;

			$pointer += $chunksize;
			$append = true;
		}	

		return $uploads['url'] . "/" . $filename; 
	} # end of WriteCSVFile

	function WriteFileSnippet( $file, $array, $append=false){
		$writemode = 'a'; // default is to open for write new
		if (!$append) {
			if (file_exists($file)) {
				chmod($file, 0777);
				unlink($file);
			}
			$writemode = 'w';
		}

		$f = fopen($file, $writemode);
		foreach ($array as $line) {
			fputcsv($f, $line);
		}

		fclose($f);
	} # end of WriteFileSnippet

	function remove_row_cols( $row, $delete_cols=array()) {

		$default_delete_keys = array('id', 'bid', 'active','deleted', 'created', 'modified', 'update_code', 'update_time') ;
		if (!is_array($delete_cols)) {
			$delete_cols = array();
		}	

		$delete_keys = array_merge( $default_delete_keys, $delete_cols);

		$newrow = array();
		foreach ($row as $key=>$cell){
			if (in_array($key, $delete_keys )) {
				continue;
			}
			$newrow[$key] = $cell;
		}
		return $newrow;
	}

	function WriteLargeExcel($pArray, $filename, $sheetnames='') {

		if (file_exists(__DIR__. "/xlsxwriter.class.php")){
			include_once(__DIR__. "/xlsxwriter.class.php");
			$start = microtime(true);

			if (empty($sheetnames) || !is_array($sheetnames)) {
				$sheetnames = array('Sheet1');
			} else {
				if (is_array($sheetnames)) {
				}
			}

			$writer = new XLSXWriter();

			foreach ($sheetnames as $sheetname) {

				$objectArray = false;

				$array = !empty($pArray[$sheetname]) ?  $pArray[$sheetname] : $pArray;
				unset($pArray[$sheetname]);
		
				$headerrow = reset($array);
				if ($headerrow instanceof stdClass){
					$objectArray = true;
					$header = new stdClass;
					#$headerrow = json_decode(json_encode($headerrow), true);
					foreach ($headerrow as $key=>$property) {
						$header->{$key} = 'string';
					}
					$headers = json_decode(json_encode($header), true);
				} else {
					$headerkeys = array_keys($headerrow);
					$headers = array();
					foreach ($headerkeys as $header) {
						#$headers[$header] = $header;
						$headers[$header] = 'string';
						#	$header;
					}
				}

				$headers = $this->remove_row_cols($headers);
				$writer->writeSheetHeader($sheetname, $headers );
				unset($headers);

				foreach ($array as $row) {
					if ($objectArray) {
						$row = json_decode(json_encode($row), true);
					}
					$row = $this->remove_row_cols($row);
					$writer->writeSheetRow($sheetname, array_values($row) );
				}
				unset($array);
			}

			$writer->writeToFile($filename);
			$end = microtime(true);
			$this->runtime = 'Run time = ' . number_format(($end - $start), 2) . ' seconds';
			$this->memory =  '#'.floor((memory_get_peak_usage())/1024/1024)."MB peak memory used "."\n";
			#echo '#'.floor((memory_get_peak_usage())/1024/1024)."MB"."\n";
			return $filename; 

		} else {
			$errors = new WP_Error();
			$errors->add('missing_file', 'Missing file: xlsxwriter.class.php. Check this is located in the correct folder');
			return $errors;
		}
		return false;
	}


} # end of OpenSpoutExcel








?>