<?php



// define an internal list of DB tables to allow shortened naming convention

// full table names may still be passed 

global $nuf_db_tables;

$nuf_db_tables = $cpg_db_tables;  // this must be configured once per project 



// This class also uses the $wpdb Wordpress database object



#####################################################################################################

// D B O B J E C T    C L A S S

#####################################################################################################

// this is the Base classes that is extended by all the 'client' classes

#[\AllowDynamicProperties]
class nssDBObjectClass {

	var $id = -99;

	var $parent = 0;

	var $parentfield = '';

	var $basetable = '';

	var $specialFields = array();

	var $basespecialFields = array( "created", "modified", 'basetable', 'debug_query', '_composite', 'basespecialFields', 'specialFields', 'tablenames', 'parent', 'parentfield', 'zoneTag', 't', 'action' );

	var $debug_query = false;

	var $_composite = array();



	var $tablenames = array();



	var $audit_enabled = 0;



	var $updatedby = '';

	var $currentUser = '';



	function __construct( $id='') {



		$this->currentUser = get_current_user_id();



		if (is_numeric($id)) {

			if ($id<0) {

				// new Client - setup defaults

				$this->createNew();

				$this->id = -1; // so we now it was definitely requested as new

			} else {

				$this->id = $id;

				$this->load();

			}

		} elseif (is_array($id)) {

			$this->loadByParams( $id );

		} elseif (is_string($id)) {

			if ($id=$this->findkeywords($id)) {

				$this->id = $id;

				$this->load();

			}

		} else {

			$this->id = false;

		}

	}



	function parent( $parentvalue='' ) {

		if (!empty($this->parentfield)) {

			$this->{$this->parentfield} = $parentvalue;

		}

	}



	function AsArray() {

		$array = json_decode(json_encode($this), true);

		if (empty($this->id) || $this->id<0) {

			//$array = array();

		}

		return $array;

	}



	function DebugPrint( $format=true, $include_tables = false ) {

		$array = $this->AsArray();



		if (!$include_tables) {

			unset($array['tablenames']);

		}

		ob_start();

		if ($format) { echo "<pre>"; }

		print_r($array);

		if ($format) { echo "</pre>"; }

		$debug = ob_get_clean();

		return $debug;

	}



	function isValid() {

		if (!empty($this->id)) { 

			return true;

		}

		return false;

	}



	function isNew() {

		if (!empty($this->id) && $this->id<0) { 

			return true;

		}

		return false;

	}



	function getAllTableNames() {

		global $wpdb;

		if (empty($GLOBALS['nuf_tablenames'])) {

			// makes a list of available tables in the DB

			$mytables=$wpdb->get_results("SHOW TABLES");

			$mytables = json_decode(json_encode($mytables), true);

			$tables = array();

			foreach ($mytables as $row) {

				foreach ($row as $key=>$mytable) {

					$tables[] = $mytable;

				}

			}

			$GLOBALS['nuf_tablenames'] = $tables;

		}

		$this->tablenames = $GLOBALS['nuf_tablenames'];

	}



	function getTableName( $table='') {

		global $wpdb;

		global $nuf_db_tables;



		if (empty($table)) {

			return false;

		}



		if (strpos($table, $wpdb->prefix)!==false) {

			// prefix is already in name so must be valid table

			return $table;

		} else {

			// we need to check if table exists with or without a prefix

			if (empty($this->tablenames)) {

				// we cache this so we only run once per object

				$this->getAllTableNames();

			}

			$tables = $this->tablenames;



			if (in_array($table, $tables)) {

				return $table;

			} elseif (in_array($wpdb->prefix . $table, $tables)) {

				return $wpdb->prefix . $table;

			} elseif (in_array($wpdb->prefix . $nuf_db_tables[$table], $tables)) {

				return $wpdb->prefix . $nuf_db_tables[$table];

			}

		}

		return false;

	}



	function loadDBObject( $table = '', $id='-1', $fieldname='id') {

		// override this 

		global $wpdb;



		if (empty($table)) {

			return false;

		}

		$tablename = $this->getTableName($table) ; 



		$params = array( $id) ;

		if (is_numeric($id)) {

			$sql = "select * from $tablename where 1=1 and $fieldname=%d";

		}  else {

			$sql = "select * from $tablename where 1=1 and $fieldname=%s";

		}

		$prepared_query = $wpdb->prepare($sql, $params);

		if ($this->debug_query) {

			echo $prepared_query;

		}

		$myrows = $wpdb->get_results($prepared_query);

		$row_arrays = json_decode(json_encode($myrows), true);



		if (empty($row_arrays)) {

			$this->id = false;

		}

		return $row_arrays;

	}



	function loadDBObjectByParams( $table = '', $fields = array() ) {

		// override this 

		global $wpdb;



		if (empty($table)) {

			return false;

		}

		$tablename = $this->getTableName($table) ; 

		$sql = "select * from $tablename where 1=1 ";





		$params = array() ;

		if (empty($fields) || !is_array($fields)) {

			$fields = array() ;

		}

		foreach ($fields as $key=>$field) {

			if (empty($key) || is_numeric($key)) {

				continue;

			}

			$sql .= " and $key = %s ";

			$params[] = $field;

		}



		$prepared_query = $wpdb->prepare($sql, $params);

		if ($this->debug_query) {

			echo $prepared_query;

		}

		$myrows = $wpdb->get_results($prepared_query);

		$row_arrays = json_decode(json_encode($myrows), true);

		if (empty($row_arrays)) {

			$this->id = false;

		}

		return $row_arrays;

	}



	function load() {

		$result = array( );



		if (empty($this->id)) {

			return false;

		}



		$results = $this->loadDBObject($this->basetable, $this->id);



		if (empty($results) && $this->id>0) {

			// did not find this record - so reset to false - should be done already by loadobject

			$this->id = false;

		}



		if (!empty($results) && is_array($results[0])) {

			if (count($results)>1) {

				// warning - possible duplicate records

			}

			$result = array_shift($results);

		}



		if (!empty($result)) {

			foreach ($result as $fieldname=>$field) {

				#echo "<br>key: " . $fieldname . "==" .$field;

				$this->{$fieldname} = stripslashes($field);

				$jdecode = json_decode(trim(stripslashes($field), '"'), true) ;

				if ($jdecode!=null && is_array($jdecode)) {

					#echo "<br>";

					#print_r($field);

					#print_r($jdecode);

					$this->{'_'.$fieldname} = $jdecode;

					$this->{$fieldname} = array();

					foreach ($jdecode as $fieldname2=>$value ) {

						#echo "<br>$fieldname2 => $value ";

						$this->{$fieldname}[$fieldname2] = !is_array($value) ? stripslashes($value) : $value;

						if (!isset($this->{$fieldname2})) {

							$this->{$fieldname2} = !is_array($value) ? stripslashes($value) : $value;

						}

					}

					if ($fieldname=='composite') {

						$this->{$fieldname} = array();

					}

				}

			}

		}

		#print_r($this->_composite);



		return false;

	}



	function loadByParams( $params = array() ) {

		$results = $this->loadDBObjectByParams($this->basetable, $params) ;



		if (!empty($results) && is_array($results[0])) {

			if (count($results)>1) {

				// warning - possible duplicate records

			}

			$result = array_shift($results);

			$this->id = $result['id'] ;

		} else {

			$this->id = false;

		}





		return $this->load(); // it is slower but less maintainenance this way

	}



	function getDBColumns($table='', $names_only = false) {

		global $wpdb;



		if (empty($table)) {

			return false;

		}



		$tablename = $this->getTableName($table) ; 



		if ($names_only) {

			$columns = $wpdb->get_col( "DESC " . $tablename, 0 );

		} else {

			// get the full table information

			$myrows = $wpdb->get_results("DESC " . $tablename);

			$columns = json_decode(json_encode($myrows), true);

		}

		#print_r($columns);



		#foreach ( $columns as $column_name ) {

		#	print_r( $column_name );

		#}



		return $columns;

	}

	function getTableSchema($tablename) {

		global $wpdb;



		if (empty($tablename)) {

			return false;

		}



		// get the full table information

		$myrows = $wpdb->get_results("SELECT * from INFORMATION_SCHEMA. COLUMNS where table_schema in (select database()) and table_name = '$tablename'");

		$columns = json_decode(json_encode($myrows), true);



		return $columns;

	}



	function createNew() {

		$columns = $this->getDBColumns($this->basetable);



		foreach ($columns as $key=>$column) {

			$columnname = $column['Field'];

			if ($columnname=='active') {

				$this->{$columnname} = 1;

			} elseif ($columnname=='deleted') {

				$this->{$columnname} = 0;

			} else {

				$this->{$columnname} = ''; // make an empty property

			}

		}

	}



	function saveDBObject( $table='', $id='-1', $fieldname='id', $objectSpecialFields=array(), $parent=0 ) {

		global $wpdb;



		$objectSaveList = array();



		$specialFields = $this->basespecialFields;

		if (!empty($objectSpecialFields)) {

			$specialFields = array_merge($objectSpecialFields,  $specialFields);

		}



		$recordid = '';



		$tablename = $this->getTableName($table) ; 

		$columns = $this->getDBColumns( $table); 



		$columnnames = array();

		$columntypes = array();

		foreach ($columns as $key=>$column) {

			$columnnames[$key] = $column['Field'];

			$columntypes[$key] = $column['Type'];

		}





		$data = array();

		if (in_array( 'composite', $columnnames)) {

			$data['composite'] = $this->_composite; // ensures we keep all original records unless overridden  ]

			#print_r($data['composite']);

		}

		foreach ($this as $key => $value) {

			#   echo "<br>$key => $value \n";

			if (in_array($key, $columnnames)  &&  !in_array($key, $specialFields) && $key!='composite') {

				#echo "<br>$key => $value \n";

				$key2 = array_search($key, $columnnames) ;

				#echo "<br>type =  " . $columntypes[$key2];

				if (is_object($value)) {

					$objectSaveList[$key] = $value;

					continue;

				} elseif (is_array($value)) {

					// dont save here now - we may be able to serialize the object

					$value  = json_encode($value);

				} elseif (is_string($value) || is_numeric($value)) {

					// do nothing use value

				}

				if ($columntypes[$key2]=='int(1)'){ 

					#echo $columnnames[$key2];

					$data[$key] = !empty($value) ? 1 : 0 ;

				} else {

					$data[$key] = $value; 

				}

			} elseif (empty($this->composite)) {

				// we have not been supplied with a composite so build one for all extra fields not already in this table

				if (in_array( 'composite', $columnnames) && !in_array( $key, $columnnames) &&  !in_array($key, $specialFields) && substr($key,0,1)!='_' && !is_numeric($key)) {

					#echo "<br>real $key => $value \n";

					if (!is_array($value)) {

						#echo "<br>$key => $value \n";

						$data['composite'][$key] = stripslashes($value);

					} else{ 

						$data['composite'][$key] = json_encode($value);

					}

				}

			} elseif (!empty($this->composite)) {

				if (is_array($this->composite)) {

					$data['composite'] = $this->composite;

				} else {

					$data['composite'] = stripslashes($this->composite);

				}

			}

       }



	   if (!empty($data['composite'])) {

		   if (is_array($data['composite']) || json_decode($data['composite'], true)===null) {

				$data['composite'] = json_encode( $data['composite'] ); 

		   }

	   }



		$where = '';

		if (!is_array($fieldname)) {

			#echo $id . " == " . $fieldname ;

			if ($id>0) {

				$recordid = $id;

				$where = array( $fieldname => $data[$fieldname] );

			}

			unset($data[$fieldname]);

		}



		$dte = new DateTime();

		if (in_array("modified", $columnnames))  {

			// update the last modified timestamp

			$data['modified'] = $dte->format("Y-m-d H:i:s");

		}

		if (in_array("created", $columnnames) && empty($where))  {

			//  only put created timestamp for new records

			$data['created'] = $dte->format("Y-m-d H:i:s");

		}



		if ($this->debug_query) {

			print_r($where);

			print_r($data);

		}



		$wpdb->show_errors();



		$this->error_data = array();

		$error_data = $this->process_fields( $tablename, $data, null ); 



		if (!empty($error_data)) {

			$this->error_data = $error_data;

		}



		if (!empty($where)) {

			$result = $wpdb->update($tablename, $data, $where) ;

			$last_query = $wpdb->last_query ;

			if (!$result) {

				#print_r($wpdb->last_error);

				if (!empty($this->error_data)) {

					echo "<div>The data entered for the following fields may be too long: <strong>" . implode(", ", $this->error_data) . "</strong></div>";

					do_action("nss_alert_developer", "update class failed", $this);

				}

			} else {

				$lastid = $where['id'] ;

				$result = $lastid;

			}

		} else {

			#print_r($converted_data);



			$result = $wpdb->insert($tablename, $data) ;

			$last_query = $wpdb->last_query ;

			if (!$result) {

				#print_r($wpdb->last_error);

				if (!empty($this->error_data)) {

					echo "<div>The data entered for the following fields may be too long: <strong>" . implode(", ", $this->error_data) . "</strong></div>";

					do_action("nss_alert_developer", "insert class failed", $this);

				}

			}

			$lastid = $wpdb->insert_id;

		}

		if ($this->audit_enabled) {

			$this->audit_log( $last_query, !empty($result), $lastid);

		}



		if ($this->debug_query) {

			if($wpdb->last_error !== '') {

				$wpdb->print_error();

			}	

		}



		if ($result==false) {

			return false;

		} else {

			if (!empty($lastid)) {

				$recordid = $lastid;

			} else {

				$recordid = $this->id;

			}

		}

		if (!empty($objectSaveList)) {

			foreach ($objectSavelist as $key=>$value) {



			}

		}

		#print_r($data);

		#print_r($where);



		return $recordid;

	}



	function audit_log( $last_query, $success=false, $lastid='' ) {

		if (empty($this->currentUser)){

			$this->currentUser = get_current_user_id();

		}

		$params = array();

		$params['query'] = $last_query;

		$params['success'] = $success;

		$params['table'] = $this->basetable;

		$params['tablename'] = $this->getTableName($this->basetable);

		$params['index'] = $this->id<0 ? $lastid : $this->id;

		$params['user'] = $this->currentUser;

		do_action("nuf_db_audit_entry", $params) ;

	}



	function process_fields( $tablename, $data, $format ) {

		global $wpdb;



		$schema = $this->getTableSchema($tablename);



		$error_fields = array();

		if (!empty($schema) && is_array($schema)) {

			foreach ($schema as $row) {

				$fieldname = $row['COLUMN_NAME'];

				if (in_array($row['DATA_TYPE'], array('varchar', 'text', 'mediumtext', 'longtext', 'tinytext')) ){

					//string  = check the lenth

					if (!empty($data[$fieldname]) && strlen($data[$fieldname])>$row['CHARACTER_MAXIMUM_LENGTH']) {

						$error_fields[] = $fieldname;

					}

				}

			}

		}



		if (!empty($error_fields)) {

			return $error_fields;

		}



		return false;



	}



	function save( $save_user = true ) {

		if ($save_user) {

			// default is to save user with the record, but can be overridden for system saved records

			$user_ID = get_current_user_id(); 



			$this->updatedby = $user_ID;

		}



		$result = $this->saveDBObject($this->basetable, $this->id,  'id', $this->specialFields) ;



		if ($result && $this->id<0) {

			$this->id = $result;

		}

		return $result;

	}



	function PushAPIError( $error ) {



		if (isset($GLOBALS['SIGAPI']['last_error'])) {

			unset($GLOBALS['SIGAPI']['last_error']);

		}

		

		$GLOBALS['SIGAPI']['last_error'] = $error;



		// we might add an error logging mechanism here

	}



	function GetLastAPIError( $echo = false ) {

		if (isset($GLOBALS['SIGAPI']['last_error'])) {

			if ($echo) {

				print_r($GLOBALS['SIGAPI']['last_error']);

			} else {

				return 	$GLOBALS['SIGAPI']['last_error'];

			}

		}

		return false;

	}



	function findkeywords( $keyword = '') {

		global $wpdb;



		$tablename = $this->getTableName($this->basetable) ; 

		$params = array();



		$sql = '';



		switch ($keyword) {

			case 'last':

				$sql = "select id from $tablename where id=(select MAX(id) from $tablename)";

				break;

			default:

				// do nothing

		}

		$prepared_query = $wpdb->prepare($sql, $params);

		if ($this->debug_query) {

			echo $prepared_query;

		}

		$myrows = $wpdb->get_results($prepared_query);

		$row_arrays = json_decode(json_encode($myrows), true);

		if (!empty($row_arrays[0]['id'])) {

			$this->id = $row_arrays[0]['id'];

			return $this->id;

		}



		return false;

	}

}



#####################################################################################################

// C L I E N T    C L A S S

#####################################################################################################

class cpgDiagnosticLogClass extends nssDBObjectClass {
	var $basetable = 'diagnostics';
	var $debug_query = false;

	function save($save_user = true) {

		if (empty($this->recordtime)) {
			$dte = new DateTime;
			$this->recordtime = $dte->format("Y-m-d H:i:s");
		}
		return parent::save($save_user);
	}
} # end of class llsDiagnosticLogClass

#################################################
## not currently in use?
#################################################
class InputXferClass extends nssDBObjectClass {
	var $clientid = 0;

	var $basetable = 'input_transfer';

	var $debug_query = false;



	function __construct($id='-1', $parent=0) {

		parent::__construct( $id, $parent );

	}
} # end of InputXferClass
class ImportXferClass extends nssDBObjectClass {
	var $clientid = 0;
	var $basetable = 'import_table1';
	var $debug_query = false;

	function __construct($id='-1', $parent=0) {
		parent::__construct( $id, $parent );
	}

} # end of InportXferClass

class ImportXferClass2 extends nssDBObjectClass {

	var $clientid = 0;

	var $basetable = 'import_table2';

	var $debug_query = false;

	function __construct($id='-1', $parent=0) {

		parent::__construct( $id, $parent );
	}
} # end of InportXferClass2
#################################################

class ImportXferClassA extends nssDBObjectClass {
	var $clientid = 0;
	var $basetable = 'import_tableA';
	var $debug_query = false;

	function __construct($id='-1', $parent=0) {
		parent::__construct( $id, $parent );
	}
} # end of InportXferClassA

class ImportXferClassB extends nssDBObjectClass {
	var $clientid = 0;
	var $basetable = 'import_tableB';
	var $debug_query = false;

	function __construct($id='-1', $parent=0) {
		parent::__construct( $id, $parent );
	}
} # end of InportXferClassB


class BatchItemUpdateClass extends nssDBObjectClass {

	var $clientid = 0;

	var $basetable = 'batch_output';

	var $debug_query = false;

	function __construct($id='-1', $parent=0) {
		parent::__construct( $id, $parent );
	}

	function __myclone()	{
		foreach ($this as $key => $val) {
			if (is_object($val) || (is_array($val))) {
				$this->{$key} = clone $val;
			}
		}
	}
} # end of BatchItemUpdateClass

class BatchItemClass extends InputXferClass {

    var $starttime = 0;

    var $endtime = 0;

	function __construct($id='-1', $parent=0) {

		parent::__construct( $id, $parent );
		
		$this->setupStartFinish() ;
	}
		
	function setupStartFinish() {
		$this->starttime = getBatchTime($this->SetupStartDate, $this->SetupStartTime);
		$this->endtime = getBatchTime($this->RunEndDate, $this->RunEndTime);
	}


	function getEndTime( $astart='') {

        if (!empty($astart) && is_a($astart, "DateTime")) {
			$this->starttime = $astart;
		}

        // add the duration to get the end time
		$endtime = new DateTime($this->starttime->format("Y-m-d H:i:s"));
        echo "<br>" . $endtime->format("Y-m-d H:i");
		echo "<br>" . $this->RunTime + $this->SetupTime;
		echo "<br>+" . ($this->RunTime + $this->SetupTime) . " hour";
		

        $endtime->modify("+" . intval(($this->RunTime + $this->SetupTime) * 60) . " minute");

        echo "<br>" . $this->starttime->format("Y-m-d H:i");
        echo "<br>" . $endtime->format("Y-m-d H:i"); 
		$this->endtime = $endtime;

		return $endtime;

	}
			 
	function RecalculateProcess( $params = array() ) {
		
		$starttime = $params['start'];
		$this->starttime = $starttime;
		
		$endtime = $this->getEndTime();

		return $endtime;
	}

} # end of BatchItemClass


class BatchClass extends nssDBObjectClass {

    var $starttime = 0;
	var $endtime = 0;

    var $processes = array();
			 
	var $DocNum = '';
	var $MOLineId = '';
	var $ItemDescription = '';
	var $ItemCode = '';
	var $Batch_Number = '';

    function __construct( $batch=''){

        if (is_array($batch)) {

            foreach ($batch as $batchitem){

                if (empty($batchitem->starttime)){

                    $batchitem->starttime = getBatchTime($batchitem->SetupStartDate, $batchitem->SetupStartTime);

                   # print_r(splitBatchTime( $this->starttime, false));
                }

                $batchitem->getEndTime($batchitem->starttime);

                $this->process[] = $batchitem;

            }

        }

    }
		
	function RecalculateBatch() {
		echo "Recalculate Batch";
		$time = $this->starttime;

		foreach ($this->processes as $batchitem) {
			echo "<br>". $batchitem->WorkCentre . ' ' . $time->format("Y-m-d H:i");
			$params = array();
			$params['start'] = $time;
			$time = $batchitem->RecalculateProcess( $params ) ;
		}

 		echo "<br>". $time->format("Y-m-d H:i");
		return $time;
	}

} # end of BatchClass



?>