<?php
/**
 Plugin        -      Name:       Personalize Login
 Description:       A plugin that replaces the WordPress login flow with a custom page.
 Version:           1.0.0
 Author:            Jarkko Laine
  License:          GPL-2.0+
 Text Domain:       personalize-login
 */
 if (file_exists(__DIR__ .  "/nuf_user_class.php")) {
	 include(__DIR__ .  "/nuf_user_class.php");
 }

 if (!function_exists("logtext")) {
	 function logtext($text='') {
		//dummy routine in case logger has not been linked in
	 }
 }
 
class Personalize_Login_Handler {
     /**
     * Initializes the plugin.
     *
     * To keep the initialization fast, only add filter and action
     * hooks in the constructor.
     */
    public function __construct() {

		// these are the login forms action and filters
		add_shortcode( 'wp_login_form ', array( $this, 'render_login_form' ) );    
		add_shortcode( 'nuf-custom-login-form', array( $this, 'render_login_form' ) );    
		add_shortcode( 'nuf_custom_login_form', array( $this, 'render_login_form' ) );    
		add_filter( 'authenticate', array( $this, 'maybe_redirect_at_authenticate' ), 101, 3 );

		$page = get_page_by_path(  'member-login' , OBJECT );
		if (isset($page)) {
			// only redirect if page exists!!
			add_action( 'login_form_login', array( $this, 'redirect_to_custom_login' ) );
		}
		add_action( 'wp_logout', array( $this, 'redirect_after_logout' ) );
		add_filter( 'login_redirect', array( $this, 'redirect_after_login' ), 10, 3 );

		// below are the registration actions and filters
		add_shortcode( 'nuf-custom-register-form', array( $this, 'render_register_form' ) );
		add_action( 'login_form_register', array( $this, 'redirect_to_custom_register' ) );
		add_action( 'login_form_register', array( $this, 'do_register_user' ) );
		// anti spam settings
		add_filter( 'admin_init' , array( $this, 'register_settings_fields' ) );
		add_action( 'wp_print_footer_scripts', array( $this, 'add_captcha_js_to_footer' ) );

		// forgotten/lost password
		add_action( 'login_form_lostpassword', array( $this, 'redirect_to_custom_lostpassword' ) );
		add_shortcode('nuf-custom-password-lost-form', array( $this, 'render_password_lost_form' ) );
		add_action( 'login_form_lostpassword', array( $this, 'do_password_lost' ) );
		add_filter( 'retrieve_password_message', array( $this, 'replace_retrieve_password_message' ), 10, 4 );

		add_action( 'login_form_rp', array( $this, 'redirect_to_custom_password_reset' ) );
		add_action( 'login_form_resetpass', array( $this, 'redirect_to_custom_password_reset' ) );
		add_shortcode('nuf-custom-password-reset-form', array( $this, 'render_password_reset_form' ) );
		add_action( 'login_form_rp', array( $this, 'do_password_reset' ) );
		add_action( 'login_form_resetpass', array( $this, 'do_password_reset' ) );

		#add_action( 'user_register', array($this, 'auto_login_new_user') );
		add_action( 'template_redirect', array( $this, 'redirect_to_protect_account_pages' ) );


		logtext("construct completed");
	}

	function redirect_to_protect_account_pages() {
		if ( ! is_user_logged_in() && is_page( array( 'member-account' ,  'profile-page') ) ) {
			$this->redirect_to_custom_login();
			exit; 
		}
	}

	function render_login_form( $attributes, $content = null ) {
		// Parse shortcode attributes
		$login_form = 'login_form';

		if (!empty($attributes['alternate_form'])) {
			$login_form = $attributes['alternate_form']; // have to do this as param will be stripped from Attributes
		}

		$default_attributes = array( 'show_title' => false, 'redirect' => '' );
		$attributes = shortcode_atts( $default_attributes, $attributes );
		$show_title = $attributes['show_title'];
 
		// Check if the user just registered
		$attributes['registered'] = isset( $_REQUEST['registered'] );
		
		if ( is_user_logged_in() ) {
			return __( 'You are already signed in.', 'personalize-login' );
		}
     
		// Pass the redirect parameter to the WordPress login functionality: by default,
		// don't specify a redirect, but if a valid redirect URL has been passed as
		// request parameter, use it.
		#$attributes['redirect'] = '';
#print_r($attributes);

		if ( isset( $_REQUEST['redirect_to'] ) ) {
			$attributes['redirect'] = wp_validate_redirect( $_REQUEST['redirect_to'], $attributes['redirect'] );
		}
  
		// Error messages
		$errors = array();
		if ( isset( $_REQUEST['login'] ) ) {
			$error_codes = explode( ',', $_REQUEST['login'] );
 
			foreach ( $error_codes as $code ) {
				$errors []= $this->get_error_message( $code );
			}
		}
		$attributes['errors'] = $errors;

		// Check if the user just requested a new password 
		$attributes['lost_password_sent'] = isset( $_REQUEST['checkemail'] ) && $_REQUEST['checkemail'] == 'confirm';

		// Check if user just updated password
		$attributes['password_updated'] = isset( $_REQUEST['password'] ) && $_REQUEST['password'] == 'changed';

		// Render the login form using an external template
		return $this->get_template_html( $login_form, $attributes );
	}


	function plugin_activated() {
		// Information needed for creating the plugin's pages
		$page_definitions = array(
			'member-login' => array(
				'title' => __( 'Sign In', 'personalize-login' ),
			   'content' => '[nuf-custom-login-form]'
			),
			'member-account' => array(
				'title' => __( 'Your Account', 'personalize-login' ),
				'content' => '[nuf-account-info]'
			),
			'member-register' => array(
				'title' => __( 'Register', 'personalize-login' ),
				'content' => '[nuf-custom-register-form]'
			),
			'member-password-lost' => array(
				'title' => __( 'Forgot Your Password?', 'personalize-login' ),
				'content' => '[nuf-custom-password-lost-form]'
			),
			'member-password-reset' => array(
				'title' => __( 'Pick a New Password', 'personalize-login' ),
				'content' => '[nuf-custom-password-reset-form]'
			)
		);
 
		foreach ( $page_definitions as $slug => $page ) {
			// Check that the page doesn't exist already
			$query = new WP_Query( 'pagename=' . $slug );
			if ( ! $query->have_posts() ) {
				// Add the page using the data from the array above
				wp_insert_post(
					array(
						'post_content'   => $page['content'],
						'post_name'      => $slug,
						'post_title'     => $page['title'],
						'post_status'    => 'publish',
						'post_type'      => 'page',
						'ping_status'    => 'closed',
						'comment_status' => 'closed',
					)
				);
			}
		}
	}


	/* Renders the contents of the given template to a string and returns it.
	*
	* @param string $template_name The name of the template to render (without .php)
	* @param array  $attributes    The PHP variables for the template
	 *
	* @return string               The contents of the template.
	*/
	function get_template_html( $template_name, $attributes = null ) {
		if ( ! $attributes ) {
			$attributes = array();
		}
 
		ob_start();
 
		do_action( 'personalize_login_before_' . $template_name );
 
		require( __DIR__  . '/login_templates/' . $template_name . '.php');
 
		do_action( 'personalize_login_after_' . $template_name );
 
		$html = ob_get_contents();
		ob_end_clean();
 
		return $html;
	}

	/**
	* Finds and returns a matching error message for the given error code.
	*
	* @param string $error_code    The error code to look up.
	*
	* @return string               An error message.
	*/
	function get_error_message( $error_code ) {
		switch ( $error_code ) {
			case 'empty_username':
				return __( 'You need to enter an email address or username', 'personalize-login' );
 
			case 'empty_password':
				return __( 'You need to enter a password to login.', 'personalize-login' );
 
			case 'invalid_username':
				return __(
					"We don't have any users with that email address. Maybe you used a different one when signing up?",
					'personalize-login'
				);
 
	        case 'incorrect_password':
		        $err = __(
			        "The password you entered wasn't quite right. <a href='%s'>Did you forget your password</a>?",
				    'personalize-login'
				);
				return sprintf( $err, wp_lostpassword_url() );
			case 'email':
				return __( 'The email address you entered is not valid.', 'personalize-login' );
 
			case 'email_exists':
				return __( 'An account exists with this email address.', 'personalize-login' );

			case 'existing_user_login':
				return __( 'An account exists with this user login.', 'personalize-login' );
 
 			case 'email_mismatch':
				return __( 'The email and repeated email are different', 'personalize-login' );

			case 'closed':
				return __( 'Registering new users is currently not allowed.', 'personalize-login' ); 


			case 'captcha':
				return __( 'The Google reCAPTCHA check failed. Are you a robot?', 'personalize-login' );


				// Lost password
 
			case 'empty_username':
				return __( 'You need to enter your email address to continue.', 'personalize-login' );
 
			case 'invalid_email':
			case 'invalidcombo':
				return __( 'There are no users registered with this email address.', 'personalize-login' );


				// Reset password
 
			case 'expiredkey':
			case 'invalidkey':
				return __( 'The password reset link you used is not valid anymore.', 'personalize-login' );
 
			case 'password_reset_mismatch':
				return __( "The two passwords you entered don't match.", 'personalize-login' );
     
			case 'password_reset_empty':
				return __( "Sorry, we don't accept empty passwords.", 'personalize-login' );

			case 'account_disabled':
				return __( "Sorry, your account is disabled. Please contact the subscriptions manager for assistance.", 'personalize-login' );

			default:
				break;

		}
     
		return __( 'An unknown error occurred. Please try again later.', 'personalize-login' );
	}

	/**
	 * Redirects the user to the correct page depending on whether he / she
	* is an admin or not.
	*
	* @param string $redirect_to   An optional redirect_to URL for admin users
	*/
	function redirect_logged_in_user( $redirect_to = null ) {
		$user = wp_get_current_user();

		if ( user_can( $user, 'manage_options' ) ) {
			if ( $redirect_to ) {
				wp_safe_redirect( $redirect_to );
			} else {
				wp_redirect( admin_url() );
			}
		} else {
			wp_redirect( home_url( 'member-account' ) );
		}
	} 
	
	/**
	* Redirect the user to the custom login page instead of wp-login.php.
	*/
	function redirect_to_custom_login() {

		#return;

		// Checks if the referrer contains the dashboard's url and assigns TRUE if so.
		$is_backend = strpos(wp_get_referer(), get_admin_url()) !== false;

		if ( $_SERVER['REQUEST_METHOD'] == 'GET' && isset($_GET['adminlogin']) && $_GET['adminlogin']!='' ) {
			return false;
		}


		if ( $_SERVER['REQUEST_METHOD'] == 'GET' && !$is_backend ) {
			$redirect_to = isset( $_REQUEST['redirect_to'] ) ? $_REQUEST['redirect_to'] : null;

			if ( is_user_logged_in() ) {
				$this->redirect_logged_in_user( $redirect_to );
				exit;
			}
			if (empty($redirect_to)) {
				$redirect_to = home_url($_SERVER['REQUEST_URI']);
			}

			$login_url = home_url( 'member-login' );
			// The rest are redirected to the login page
			if ( ! empty( $redirect_to ) ) {
				$login_url = add_query_arg( 'redirect_to', $redirect_to, $login_url );
			}

			wp_redirect( $login_url );
			exit;
		}
	}

	/**
	* Redirect the user to the custom login page instead of wp-login.php.
	*/
	function redirect_to_custom_loginXXX() {
		if ( $_SERVER['REQUEST_METHOD'] == 'GET' ) {
			$redirect_to = isset( $_REQUEST['redirect_to'] ) ? $_REQUEST['redirect_to'] : null;
     
			if ( is_user_logged_in() ) {
				$this->redirect_logged_in_user( $redirect_to );
				exit;
			}
 
			// The rest are redirected to the login page
			$login_url = home_url( 'member-login' );
			if ( ! empty( $redirect_to ) ) {
				$login_url = add_query_arg( 'redirect_to', $redirect_to, $login_url );
			}
 
			wp_redirect( $login_url );
			exit;
		}
	}

	/**
	* Redirect the user after authentication if there were any errors.
	*
	* @param Wp_User|Wp_Error  $user       The signed in user, or the errors that have occurred during login.
	* @param string            $username   The user name used to log in.
	* @param string            $password   The password used to log in.
	*
	* @return Wp_User|Wp_Error The logged in user, or error information if there were errors.
	*/
	function maybe_redirect_at_authenticate( $user, $username, $password ) {
		// Check if the earlier authenticate filter (most likely, 
		// the default WordPress authentication) functions have found errors
		
		if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
			$disabled = get_user_meta( $user->ID, 'nuf_disable_user', true );
		
			// Is the use logging in disabled?
			if ( $disabled == '1' ) {
				// Clear cookies, a.k.a log user out
				wp_clear_auth_cookie();

				#print_r($user);
				$user = new WP_Error( 'account_disabled', 'Your account is disabled', 'my best' );

				#// Build login URL and then redirect
				#$login_url = site_url( 'wp-login.php', 'login' );
				#$login_url = add_query_arg( 'disabled', '1', $login_url );
				#wp_redirect( $login_url );
				#exit;
			}


			if ( is_wp_error( $user ) ) {
				$error_codes = join( ',', $user->get_error_codes() );
 
				$login_url = home_url( 'member-login' );
				$login_url = add_query_arg( 'login', $error_codes, $login_url );
 
 		        wp_redirect( $login_url );
	            exit;
			}
		}

		return $user;
	}

	/**
	* Returns the URL to which the user should be redirected after the (successful) login.
	*
	* @param string           $redirect_to           The redirect destination URL.
	* @param string           $requested_redirect_to The requested redirect destination URL passed as a parameter.
	* @param WP_User|WP_Error $user                  WP_User object if login was successful, WP_Error object otherwise.
	*
	* @return string Redirect URL
	*/
	function redirect_after_login( $redirect_to, $requested_redirect_to, $user ) {
		$redirect_url = home_url();
 
		if ( ! isset( $user->ID ) ) {
			return $redirect_url;
		}

 		// if the user is part of the team then redirect straight to the dashboard
 		if ( !empty( $user->roles ) && is_array( $user->roles ) ) {

			if ( in_array( 'cpgCustPTeam', $user->roles ) ) {
				// redirect them to the default place
				//$data_login = get_option('axl_jsa_login_wid_setup');
				//return get_permalink($data_login[0]);

				$redirect_url = site_url( '/dashboard/' );
				if (!empty($requested_redirect_to) && empty($redirect_url)) {
					$redirect_url = $requested_redirect_to;
				}
				#return home_url( 'dashboard' );
				#return $redirect_url;
			} 
		}

		// administrators are not redirected here!!
		if ( !empty( $user->allcaps ) && is_array( $user->allcaps ) ) { 
			if ( array_key_exists( 'cpgCustomer', $user->allcaps ) && !array_key_exists( 'administrator', $user->allcaps ) ) {
				// redirect them to the default place
				//$data_login = get_option('axl_jsa_login_wid_setup');

				//return get_permalink($data_login[0]);
				$redirect_url = home_url( 'member-account' );
				if (!empty($requested_redirect_to)) {
					$redirect_url = $requested_redirect_to;
				}
			} 
		}
 
		if ( user_can( $user, 'manage_options' ) && in_array( 'administrator', $user->roles)) {
			echo "user_can";
			// Use the redirect_to parameter if one is set, otherwise redirect to admin dashboard.
			if ( $requested_redirect_to == '' ) {
				$redirect_url = admin_url();
			} else {
				$redirect_url = $requested_redirect_to;
			}
		} elseif (!in_array( 'cpgCustPTeam', $user->roles ) ) {
			echo "user_cant";
			// Non-admin users always go to their account page after login unless directed to specific page
			// we need to intercept non-team users accessing the dashboard
			if ( $requested_redirect_to == '' ||  $requested_redirect_to == 'dashboard' ) {
				$redirect_url = home_url( 'member-account' );
			} else {
				$redirect_url = $requested_redirect_to;
			}
		}
		return wp_validate_redirect( $redirect_url, home_url() );
	}

	/**
	* Redirect to custom login page after the user has been logged out.
	*/
	function redirect_after_logout() {
		#$redirect_url = home_url( 'member-login?logged_out=true' );
		$redirect_url = home_url();
		wp_safe_redirect( $redirect_url );
		exit;
	}	

	################################################################################################################
	### Registration functions
	################################################################################################################

	/**
	* A shortcode for rendering the new user registration form.
	*
	* @param  array   $attributes  Shortcode attributes.
	* @param  string  $content     The text content for shortcode. Not used.
	*
	* @return string  The shortcode output
	*/
	function render_register_form( $attributes, $content = null ) {
		// Parse shortcode attributes
		$default_attributes = array( 'show_title' => false );
		$attributes = shortcode_atts( $default_attributes, $attributes );
		// Retrieve recaptcha key
		$attributes['recaptcha_site_key'] = get_option( 'personalize-login-recaptcha-site-key', null );
		
		if ( is_user_logged_in() ) {
			return __( 'You are already signed in.', 'personalize-login' );
		} elseif ( ! get_option( 'users_can_register')  && 1==0 ) {
			return __( 'Registering new users is currently not allowed.', 'personalize-login' );
		} else {
			// Retrieve possible errors from request parameters
			$attributes['errors'] = array();
			if ( isset( $_REQUEST['register-errors'] ) ) {
				$error_codes = explode( ',', $_REQUEST['register-errors'] );
 
				foreach ( $error_codes as $error_code ) {
					$attributes['errors'] []= $this->get_error_message( $error_code );
				}
			}
			return $this->get_template_html( 'register_form', $attributes );
		}
	}

	/**
	* Redirects the user to the custom registration page instead
	* of wp-login.php?action=register.
	*/
	function redirect_to_custom_register() {
		if ( 'GET' == $_SERVER['REQUEST_METHOD'] ) {
			if ( is_user_logged_in() ) {
				$this->redirect_logged_in_user();
			} else {
				wp_redirect( home_url( 'member-register' ) );
			}
			exit;
		}
	}

	/**
	* Validates and then completes the new user signup process if all went well.
	*
	* @param string $email         The new user's email address
	* @param string $first_name    The new user's first name
	* @param string $last_name     The new user's last name
	*
	* @return int|WP_Error         The id of the user that was created, or error if failed.
	*/
	function register_user( $email, $first_name, $last_name ) {
		$errors = new WP_Error();
 
		// Email address is used as both username and email. It is also the only
		// parameter we need to validate
		if ( ! is_email( $email ) ) {
			$errors->add( 'email', $this->get_error_message( 'email' ) );
			return $errors;
		}
 
 		if ( username_exists( $email ) || email_exists( $email ) ) {
			$errors->add( 'email_exists', $this->get_error_message( 'email_exists') );
			return $errors;
		}

		if ( username_exists( $email ) || email_exists( $email ) ) {
			$errors->add( 'email_exists', $this->get_error_message( 'email_exists') );
			return $errors;
		}
 
		// Generate the password so that the subscriber will have to check email...
		$password = wp_generate_password( 12, false );
 
		$user_data = array(
			'user_login'    => $email,
			'user_email'    => $email,
			'user_pass'     => $password,
			'first_name'    => $first_name,
			'last_name'     => $last_name,
			'nickname'      => $first_name,
		);
 
		$user_id = wp_insert_user( $user_data );
		#wp_new_user_notification( $user_id, $password );
		wp_new_user_notification( $user_id, null, true );
 
	    return $user_id;
	}
	function admin_create_new_user($username, $email, $password='', $first_name='', $last_name='', $repeatemail='') {
		$send_email = false;
		return $this->create_new_user($username, $email, $password, $first_name, $last_name, $repeatemail, $send_email);
	}

	function create_new_surveycustomer_user($username, $email, $password='', $first_name='', $last_name='', $repeatemail='', $send_email=true) {
		$userID = $this->create_new_user($username, $email, $password, $first_name, $last_name, $email, false);
			
		if (!is_wp_error($userID)) {
			// Replace the current role with 'nssCustomer' role
			$user = new WP_User( $userID );
			$user->set_role( 'nssCustomer' );
			// send the welcome acm_registration_send_password_link message
		#	if ($send_email) {
		#		$this->acm_registration_send_password_link($userID, 'nssCustomer');
		#	}
			#$this->auto_login_new_user( $userID);
			#logtext("Created new userID: " . $userID);
			return $userID;
		}
		return false;
	}


	function create_new_surveyadminteam_user($username, $email, $password='', $first_name='', $last_name='', $repeatemail='', $send_email=true) {
		$userID = $this->create_new_user($username, $email, $password, $first_name, $last_name, $email, false);
			
		if (!is_wp_error($userID)) {
			// Replace the current role with 'nssSurveyTeam' role
			$user = new WP_User( $userID );
			$user->set_role( 'nssSurveyTeam' );
			// send the welcome acm_registration_send_password_link message
			if ($send_email) {
				$this->acm_registration_send_password_link($userID, 'nssSurveyTeam');
			}
			#$this->auto_login_new_user( $userID);
			#logtext("Created new userID: " . $userID);
			return $userID;
		}
		return false;
	}

	function create_new_user($username, $email, $password='', $first_name='', $last_name='', $repeatemail='', $send_email=true) {

		if (empty($username)) {
			$username = $email;
		}
		if ($password=='') {
			$password = 'oursurveys'; // this needs to be randomised later
		}

		$errors = new WP_Error();
 
		if ( $email!= $repeatemail  ) {
			$errors->add( 'email_mismatch', $this->get_error_message( 'email_mismatch') );
			return $errors;
		}

		// Email address is used as both username and email. It is also the only
		// parameter we need to validate
		if ( ! is_email( $email ) ) {
			$errors->add( 'email', $this->get_error_message( 'email' ) );
			return $errors;
		}

		if ( username_exists( $email ) || email_exists( $email ) ) {
			$errors->add( 'email_exists', $this->get_error_message( 'email_exists') );
			return $errors;
		}

 		logtext("create_new_user");

		$userID='';
		if (!empty($username) && !empty($email) && !empty($password)) {
		    $random_password = wp_generate_password($length=12, $include_standard_special_chars=false);

			#logtext("generated random" . $random_password . "|" . $email );
			$userID = wp_create_user($username, $random_password, $email);
			logtext($userID);

			if (!is_wp_error($userID)) {
				wp_set_password($password, $userID);
				if (!empty($first_name) && !empty($last_name)) {
					$user_data = wp_update_user( array( 'ID' => $userID, 'first_name' => $first_name, 'last_name' => $last_name ) );
				}
				#$data["result"] = SUCCESS;
				// Replace the current role with 'team' role

				$user = new WP_User( $userID );
				#$user->set_role( 'administrator' );
				$user->set_role( 'nssUser' );
				#logtext($user);
				// send the welcome acm_registration_send_password_link message
				if ($send_email) {
					$this->acm_registration_send_password_link($userID, 'nssUser' );
				}
				#$this->auto_login_new_user( $userID);
				logtext("Created new userID: " . $userID);

			}else{
			#$data["result"] = FAIL;
			}
		}else{
			#$data["result"] = FAIL;
		}
		return $userID;
	}

	function auto_login_new_user( $user_id ) {
		if (isset($_SESSION['simple_cart_id']) && $_SESSION['simple_cart_id']!='') {
			wp_set_current_user($user_id);
			wp_set_auth_cookie($user_id);
            // You can change home_url() to the specific URL,such as 
			//wp_redirect( 'http://www.wpcoke.com' );
			wp_redirect( home_url() . '/process-order/' );
			exit;
		}
    }


	/**
	* Handles the registration of a new user.
	*
	* Used through the action hook "login_form_register" activated on wp-login.php
	* when accessed through the registration action.
	*/
	function do_register_user() {
		if ( 'POST' == $_SERVER['REQUEST_METHOD'] ) {

	        $redirect_url = home_url( 'member-register' );

			if ( ! get_option( 'users_can_register' ) ) {
				// Registration closed, display error
				$redirect_url = add_query_arg( 'register-errors', 'closed', $redirect_url );
			} elseif ( ! $this->verify_recaptcha() ) {
				// Recaptcha check failed, display error
				$redirect_url = add_query_arg( 'register-$errors', 'captcha', $redirect_url );
			} else {
				$username = ''; // it will be set to email address in create_new_user
				$first_name = sanitize_text_field( $_POST['first_name'] );
				$last_name = sanitize_text_field( $_POST['last_name'] );
				$email = sanitize_text_field( $_POST['email'] );
				$repeatemail = sanitize_text_field( $_POST['repeatemail'] );
 
 				logtext("do_register");
				$result = $this->create_new_user($username, $email, $password, $first_name, $last_name, $repeatemail);

				if ( is_wp_error( $result ) ) {
					// Parse errors into a string and append as parameter to redirect
					$errors = join( ',', $result->get_error_codes() );
					$redirect_url = add_query_arg( 'register-errors', $errors, $redirect_url );
				} else {
					// Success, redirect to login page.
					logtext("do_register - no errors");

					if (isset($_SESSION['simple_cart_id']) && $_SESSION['simple_cart_id']!='') {
						// do the registration and revert to order handling.
						logtext("create with cart - redirect to process_order");
						$userID = $result;
						$this->auto_login_new_user( $userID);
						$redirect_url = home_url( 'process-order' );
					} else {
						// do the normal registration method
						logtext("created without cart - no login");
						$redirect_url = home_url( 'member-login' );
						$redirect_url = add_query_arg( 'registered', $email, $redirect_url );
					}
				}
			}
			logtext($redirect_url);
			wp_redirect( $redirect_url );
			exit;
		}
	}


	/**
	* Handles sending password retrieval email to user.
	*
	* @return bool|WP_Error True: when finish. WP_Error on error
	*/
	function acm_registration_send_password_link( $new_user_id='', $user_type='nssUser') {
		$errors = new WP_Error();

logtext("send password");

		if (empty($new_user_id)) {
			return false;
		}
		$user_data = get_user_by('ID', $new_user_id);
		logtext($user_data);
		if (empty($user_data)) {
			return false;
		}

		// Redefining user_login ensures we return the right case in the email.
		$user_login = $user_data->user_login;
		$user_email = $user_data->user_email;
		$key = get_password_reset_key( $user_data );

		if ( is_wp_error( $key ) ) {
			return $key;
		}

		$reg = '';
		switch ($user_type) {
			case 'nssSurveyTeam':
				$reg = new SurveyTeamRegistrationClass($new_user_id);
				break;
			case 'nssCustomer':
				$reg = new CustomerRegistrationClass($new_user_id);
				break;
			default:
				$reg = new UserRegistrationClass;
		}
		
		if (is_a($reg, 'UserRegistrationClass')) {

			#print_r($reg);
			$reg->recipients[] = $user_email;
			$reg->MailDoc();
			
			return false;
		}

		$message = __('Hello') . "\r\n\r\n";
		$message .= network_home_url( '/' ) . "\r\n\r\n";
		$message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
		$message .= __('To set your password and complete your registration, please click on the link below. You can then access your account and start ordering from us.') . "\r\n\r\n";
		$message .= '<' . network_site_url("wp-login.php?action=rp&key=$key&login=" . rawurlencode($user_login), 'login') . ">\r\n";
		$message .= __('The link avove is valid for 24 hours.') . "\r\n";
		$message .= __('If you are unable to complete this action in time you will need to visit the web site (link below), and then follow the forgotten password instructions to resend a new password reset link.') . "\r\n\r\n";
		$message .= '' . network_site_url("/member-login/") . "\r\n";

		$blogname = wp_specialchars_decode(get_option('blogname'), ENT_QUOTES);

		/* translators: Password reset email subject. 1: Site name */
		$title = sprintf( __('[%s] Welcome to Sussex Surveyors'), $blogname ); 
		$title = 'Welcome to Sussex Surveyors'; // override default title
	
		logtext($message);
		if ( $message && !wp_mail( $user_email, wp_specialchars_decode( $title ), $message ) )
			wp_die( __('The email could not be sent.') . "<br />\n" . __('Possible reason: your host may have disabled the mail() function.') );

		return true;
	}

	#####################################################################################################
	## CAPTCHA
	#####################################################################################################

	/**
	* Registers the settings fields needed by the plugin.
	*/
	function register_settings_fields() {
		// Create settings fields for the two keys used by reCAPTCHA
		register_setting( 'general', 'personalize-login-recaptcha-site-key' );
		register_setting( 'general', 'personalize-login-recaptcha-secret-key' );
 
		add_settings_field(
			'personalize-login-recaptcha-site-key',
			'<label for="personalize-login-recaptcha-site-key">' . __( 'reCAPTCHA site key' , 'personalize-login' ) . '</label>',
			array( $this, 'render_recaptcha_site_key_field' ),
			'general'
		);
 
		add_settings_field(
			'personalize-login-recaptcha-secret-key',
			'<label for="personalize-login-recaptcha-secret-key">' . __( 'reCAPTCHA secret key' , 'personalize-login' ) . '</label>',
			array( $this, 'render_recaptcha_secret_key_field' ),
			'general'
		);
	}
 
	function render_recaptcha_site_key_field() {
		$value = get_option( 'personalize-login-recaptcha-site-key', '' );
		echo '<input type="text" id="personalize-login-recaptcha-site-key" name="personalize-login-recaptcha-site-key" value="' . esc_attr( $value ) . '" />';
	}
 
	function render_recaptcha_secret_key_field() {
		$value = get_option( 'personalize-login-recaptcha-secret-key', '' );
		echo '<input type="text" id="personalize-login-recaptcha-secret-key" name="personalize-login-recaptcha-secret-key" value="' . esc_attr( $value ) . '" />';
	}

	/**
	* An action function used to include the reCAPTCHA JavaScript file
	* at the end of the page.
	*/
	function add_captcha_js_to_footer() {
		echo "<script src='https://www.google.com/recaptcha/api.js'></script>";
	}

	/**
	* Checks that the reCAPTCHA parameter sent with the registration
	* request is valid.
	*
	* @return bool True if the CAPTCHA is OK, otherwise false.
	*/
	function verify_recaptcha() {
		// This field is set by the recaptcha widget if check is successful
		if ( isset ( $_POST['g-recaptcha-response'] ) ) {
			$captcha_response = $_POST['g-recaptcha-response'];
		} else {
			return false;
		}
 
		// Verify the captcha response from Google
		$response = wp_remote_post(
			'https://www.google.com/recaptcha/api/siteverify',
			array(
				'body' => array(
					'secret' => get_option( 'personalize-login-recaptcha-secret-key' ),
					'response' => $captcha_response
				)
			)
		);
 
		$success = false;
		if ( $response && is_array( $response ) ) {
			$decoded_response = json_decode( $response['body'] );
			$success = $decoded_response->success;
		}
 
		return $success;
	}

	###############################################
	/**
	* Redirects the user to the custom "Forgot your password?" page instead of
	* wp-login.php?action=lostpassword.
	*/
	function redirect_to_custom_lostpassword() {
		if ( 'GET' == $_SERVER['REQUEST_METHOD'] ) {
			if ( is_user_logged_in() ) {
				$this->redirect_logged_in_user();
				exit;
			}
 
			wp_redirect( home_url( 'member-password-lost' ) );
			exit;
		}
	}

	/**
	* A shortcode for rendering the form used to initiate the password reset.
	*
	* @param  array   $attributes  Shortcode attributes.
	* @param  string  $content     The text content for shortcode. Not used.
	*
	* @return string  The shortcode output
	*/
	function render_password_lost_form( $attributes, $content = null ) {
		// Parse shortcode attributes
		$default_attributes = array( 'show_title' => false );
		$attributes = shortcode_atts( $default_attributes, $attributes );
 
	   if ( is_user_logged_in() ) {
		    return __( 'You are already signed in.', 'personalize-login' );
		} else {
			// Retrieve possible errors from request parameters
			$attributes['errors'] = array();
			if ( isset( $_REQUEST['errors'] ) ) {
				$error_codes = explode( ',', $_REQUEST['errors'] );
 
			    foreach ( $error_codes as $error_code ) {
					$attributes['errors'] []= $this->get_error_message( $error_code );
				}
			}
			return $this->get_template_html( 'password_lost_form', $attributes );
		}
	}

	/**
	* Initiates password reset.
	*/
	function do_password_lost() {
		if ( 'POST' == $_SERVER['REQUEST_METHOD'] ) {
			$errors = retrieve_password();
			if ( is_wp_error( $errors ) ) {
				// Errors found
				$redirect_url = home_url( 'member-password-lost' );
				$redirect_url = add_query_arg( 'errors', join( ',', $errors->get_error_codes() ), $redirect_url );
			} else {
				// Email sent
				$redirect_url = home_url( 'member-login' );
				$redirect_url = add_query_arg( 'checkemail', 'confirm', $redirect_url );
			}
 
			wp_redirect( $redirect_url );
			exit;
		}
	}

	/**
	* Returns the message body for the password reset mail.
	* Called through the retrieve_password_message filter.
	*
	* @param string  $message    Default mail message.
	* @param string  $key        The activation key.
	* @param string  $user_login The username for the user.
	* @param WP_User $user_data  WP_User object.
	*
	* @return string   The mail message to send.
	*/
	function replace_retrieve_password_message( $message, $key, $user_login, $user_data ) {
		// Create new message
		$msg  = __( 'Hello!', 'personalize-login' ) . "\r\n\r\n";
		$msg .= sprintf( __( 'You asked us to reset your password for your account using the email address: %s', 'personalize-login' ), $user_login ) . "\r\n\r\n";
		$msg .= __( "If this was a mistake, or you didn't ask for a password reset, just ignore this email and nothing will happen.", 'personalize-login' ) . "\r\n\r\n";
		$msg .= __( 'To reset your password, visit the following address:', 'personalize-login' ) . "\r\n\r\n";
		$msg .= site_url( "wp-login.php?action=rp&key=$key&login=" . rawurlencode( $user_login ), 'login' ) . "\r\n\r\n"; 
		$msg .= __( 'Thanks!', 'personalize-login' ) . "\r\n";
		$msg .= __( 'The  Team', 'personalize-login' ) . "\r\n";

		 
 
		return $msg;
	}

	/**
	* Redirects to the custom password reset page, or the login page
	* if there are errors.
	*/
	function redirect_to_custom_password_reset() {
		if ( 'GET' == $_SERVER['REQUEST_METHOD'] ) {
			// Verify key / login combo
			$user = check_password_reset_key( $_REQUEST['key'], $_REQUEST['login'] );
			if ( ! $user || is_wp_error( $user ) ) {
				if ( $user && $user->get_error_code() === 'expired_key' ) {
					wp_redirect( home_url( 'member-login?login=expiredkey' ) );
				} else {
					wp_redirect( home_url( 'member-login?login=invalidkey' ) );
				}
				 exit;
			}
 
			$redirect_url = home_url( 'member-password-reset' );
			$redirect_url = add_query_arg( 'login', esc_attr( $_REQUEST['login'] ), $redirect_url );
			$redirect_url = add_query_arg( 'key', esc_attr( $_REQUEST['key'] ), $redirect_url );
 
	        wp_redirect( $redirect_url );
		    exit;
		}
	}

	/**
	* A shortcode for rendering the form used to reset a user's password.
	*
	* @param  array   $attributes  Shortcode attributes.
	* @param  string  $content     The text content for shortcode. Not used.
	*
	* @return string  The shortcode output
	*/
	function render_password_reset_form( $attributes, $content = null ) {
		// Parse shortcode attributes
		$default_attributes = array( 'show_title' => false );
		$attributes = shortcode_atts( $default_attributes, $attributes );
 
	   if ( is_user_logged_in() ) {
		    return __( 'You are already signed in.', 'personalize-login' );
		} else {
			if ( isset( $_REQUEST['login'] ) && isset( $_REQUEST['key'] ) ) {
				$attributes['login'] = $_REQUEST['login'];
				$attributes['key'] = $_REQUEST['key'];
 
				// Error messages
				$errors = array();
				if ( isset( $_REQUEST['error'] ) ) {
					$error_codes = explode( ',', $_REQUEST['error'] );
 
					foreach ( $error_codes as $code ) {
						$errors []= $this->get_error_message( $code );
					}
				}
				$attributes['errors'] = $errors;
 
				return $this->get_template_html( 'password_reset_form', $attributes );
			} else {
				return __( 'Invalid password reset link.', 'personalize-login' );
			}
		}
	}

	/**
	* Resets the user's password if the password reset form was submitted.
	*/
	function do_password_reset() {
		if ( 'POST' == $_SERVER['REQUEST_METHOD'] ) {
			$rp_key = $_REQUEST['rp_key'];
			$rp_login = $_REQUEST['rp_login'];
 
	        $user = check_password_reset_key( $rp_key, $rp_login );

	        if ( ! $user || is_wp_error( $user ) ) {
		        if ( $user && $user->get_error_code() === 'expired_key' ) {
			        wp_redirect( home_url( 'member-login?login=expiredkey' ) );
				} else {
					wp_redirect( home_url( 'member-login?login=invalidkey' ) );
				}
				exit;
			}
 
	        if ( isset( $_POST['pass1'] ) ) {
		        if ( $_POST['pass1'] != $_POST['pass2'] ) {
			        // Passwords don't match
				    $redirect_url = home_url( 'member-password-reset' );
 
					$redirect_url = add_query_arg( 'key', $rp_key, $redirect_url );
					$redirect_url = add_query_arg( 'login', $rp_login, $redirect_url );
					$redirect_url = add_query_arg( 'error', 'password_reset_mismatch', $redirect_url );
 
					wp_redirect( $redirect_url );
					exit;
				}
 
				if ( empty( $_POST['pass1'] ) ) {
					// Password is empty
					$redirect_url = home_url( 'member-password-reset' );
 
	                $redirect_url = add_query_arg( 'key', $rp_key, $redirect_url );
		            $redirect_url = add_query_arg( 'login', $rp_login, $redirect_url );
			        $redirect_url = add_query_arg( 'error', 'password_reset_empty', $redirect_url );
 
				    wp_redirect( $redirect_url );
					exit;
				}
 
				// Parameter checks OK, reset password
				reset_password( $user, $_POST['pass1'] );
				wp_redirect( home_url( 'member-login?password=changed' ) );
			} else {
				echo "Invalid request.";
			}
 
			exit;
		}
	}

} # end of personlize_login_handler 

###########################################################################################
// Initialize the plugin
global $personalize_login_pages_plugin;
$personalize_login_pages_plugin = new Personalize_Login_Handler();

#print_r($personalize_login_pages_plugin);

###########################################################################################

add_action('init', 'activate_personalized_login', 100);
function activate_personalized_login() {
	global $personalize_login_pages_plugin;

	$page_name_slug = 'member-account' ; #. #1489054938345-700489da-596d' ; //this needs to be saved as a plugin option field somewhere
	$page = url_to_postid( $page_name_slug );
	if (empty($page)) {
		$personalize_login_pages_plugin->plugin_activated(); // only run this once normally !!
	}
}

##############################################################################################

final class nuf_disable_users {

	/**
	 * Initialize all the things
	 *
	 * @since 1.0.0
	 */
	function __construct() {

		// Actions
	#	add_action( 'init',                       array( $this, 'load_textdomain'             )        );
		add_action( 'show_user_profile',          array( $this, 'use_profile_field'           )        );
		add_action( 'edit_user_profile',          array( $this, 'use_profile_field'           )        );
		add_action( 'personal_options_update',    array( $this, 'user_profile_field_save'     )        );
		add_action( 'edit_user_profile_update',   array( $this, 'user_profile_field_save'     )        );
		add_action( 'wp_login',                   array( $this, 'user_login'                  ), 10, 2 );
		add_action( 'manage_users_custom_column', array( $this, 'manage_users_column_content' ), 10, 3 );
		add_action( 'admin_footer-users.php',	  array( $this, 'manage_users_css'            )        );
		
		// Filters
		add_filter( 'login_message',              array( $this, 'user_login_message'          )        );
		add_filter( 'manage_users_columns',       array( $this, 'manage_users_columns'	      )        );
	}

	/**
	 * Load the textdomain so we can support other languages
	 *
	 * @since 1.0.0
	 */
	public function load_textdomain() {
		$domain = 'nuf_disable_users';
		$locale = apply_filters( 'plugin_locale', get_locale(), $domain );
		load_textdomain( $domain, WP_LANG_DIR . '/' . $domain . '/' . $domain . '-' . $locale . '.mo' );
		load_plugin_textdomain( $domain, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}

	/**
	 * Add the field to user profiles
	 *
	 * @since 1.0.0
	 * @param object $user
	 */
	public function use_profile_field( $user ) {

		// Only show this option to users who can delete other users
		if ( !current_user_can( 'edit_users' ) )
			return;
		?>
		<table class="form-table">
			<tbody>
				<tr>
					<th>
						<label for="nuf_disable_user"><?php _e(' Disable User Account', 'nuf_disable_users' ); ?></label>
					</th>
					<td>
						<input type="checkbox" name="nuf_disable_user" id="nuf_disable_user" value="1" <?php checked( 1, get_the_author_meta( 'nuf_disable_user', $user->ID ) ); ?> />
						<span class="description"><?php _e( 'If checked, the user cannot login with this account.' , 'nuf_disable_users' ); ?></span>
					</td>
				</tr>
			<tbody>
		</table>
		<?php
	}

	/**
	 * Saves the custom field to user meta
	 *
	 * @since 1.0.0
	 * @param int $user_id
	 */
	public function user_profile_field_save( $user_id ) {

		// Only worry about saving this field if the user has access
		if ( !current_user_can( 'edit_users' ) )
			return;

		if ( !isset( $_POST['nuf_disable_user'] ) ) {
			$disabled = 0;
		} else {
			$disabled = $_POST['nuf_disable_user'];
		}
	 
		update_user_meta( $user_id, 'nuf_disable_user', $disabled );
	}

	/**
	 * After login check to see if user account is disabled
	 *
	 * @since 1.0.0
	 * @param string $user_login
	 * @param object $user
	 */
	public function user_login( $user_login, $user = null ) {

		if ( !$user ) {
			$user = get_user_by('login', $user_login);
		}
		if ( !$user ) {
			// not logged in - definitely not disabled
			return;
		}
		// Get user meta
		$disabled = get_user_meta( $user->ID, 'nuf_disable_user', true );
		
		// Is the use logging in disabled?
		if ( $disabled == '1' ) {
			// Clear cookies, a.k.a log user out
			wp_clear_auth_cookie();

			// Build login URL and then redirect
			$login_url = site_url( 'wp-login.php', 'login' );
			$login_url = add_query_arg( 'disabled', '1', $login_url );
			wp_redirect( $login_url );
			exit;
		}
	}

	/**
	 * Show a notice to users who try to login and are disabled
	 *
	 * @since 1.0.0
	 * @param string $message
	 * @return string
	 */
	public function user_login_message( $message ) {
		echo "login test";

		// Show the error message if it seems to be a disabled user
		if ( isset( $_GET['disabled'] ) && $_GET['disabled'] == 1 ) 
			$message =  '<div id="login_error">' . apply_filters( 'nuf_disable_users_notice', __( 'Account disabled', 'nuf_disable_users' ) ) . '</div>';

		return $message;
	}

	/**
	 * Add custom disabled column to users list
	 *
	 * @since 1.0.3
	 * @param array $defaults
	 * @return array
	 */
	public function manage_users_columns( $defaults ) {

		$defaults['nuf_user_disabled'] = __( 'Disabled', 'nuf_disable_users' );
		return $defaults;
	}

	/**
	 * Set content of disabled users column
	 *
	 * @since 1.0.3
	 * @param empty $empty
	 * @param string $column_name
	 * @param int $user_ID
	 * @return string
	 */
	public function manage_users_column_content( $empty, $column_name, $user_ID ) {

		if ( $column_name == 'nuf_user_disabled' ) {
			if ( get_the_author_meta( 'nuf_disable_user', $user_ID )	== 1 ) {
				return __( 'Disabled', 'nuf_disable_users' );
			}
		}
	}

	/**
	 * Specifiy the width of our custom column
	 *
	 * @since 1.0.3
 	 */
	public function manage_users_css() {
		echo '<style type="text/css">.column-nuf_user_disabled { width: 80px; }</style>';
	}
}

##########################################################################################
##########################################################################################

class nufUserCapabilitiesClass {

	var $member_admin = 'cpgCustPTeam';  // this is the administrator level
	var $standard_member = 'cpgCustomer';
	var $member_admin_role = "custp_admin";
	var $group_title  =  'Custom Pharma Team';

	function __construct() {
		add_action( 'init', array($this, 'wpsites_remove_contributor_capabilities') );

		add_action('edit_user_profile', array($this, 'check_set_user_metadata'), 10, 1 );
		add_action('show_user_profile', array($this, 'check_set_user_metadata'), 10, 1 ); // editing your own profile
		add_filter("nuf_check_authorised_user", array($this, "CheckAuthorisedUser"), 10, 1);
	}

	function CheckAuthorisedUser($dummy, $role = '') {
		global $personalize_login_pages_plugin;

		$member_admin = $this->member_admin;
		$standard_member = $this->standard_member;
		$member_admin_role = $this->member_admin_role;

		if ( is_user_logged_in() ) {

			$login_string = 'Logout';
			$link_url = wp_logout_url();

			$user = wp_get_current_user();
			$roles = ( array ) $user->roles;
			$caps = array_keys( $user->allcaps );
			// if WP administrator OR member_admin or standard member or have the role of member_admin
			if (in_array("manage_options", $caps) || in_array($member_admin, $caps) || in_array($standard_member, $caps) || in_array($member_admin_role, $caps)) {
				if (empty($role)) {
					return true;
				} else {
					// we need to check against specific role that was passed in
					if (in_array($role, $caps)) {
						return true;
					}
					return false;
				}
			} else {
				// logged in user, but does not have the correct permissions
				die("Sorry you don't have permission to access this page");
			} 
		} else {
			$login_string = 'Login';
			$link_url = '/login/';

			echo '<div style="max-width:500px;>';
			echo "login here?";

			$args = array(
			'echo' => true,
			'redirect' => '/dashboard/',
		    'form_id' => 'loginform',
		    'label_username' => __( 'Username or Email Address' ),
		    'label_password' => __( 'Password' ),
		    'label_remember' => __( 'Remember Me' ),
		    'label_log_in' => __( 'Log In' ),
		    'id_username' => 'user_login',
		    'id_password' => 'user_pass',
		    'id_remember' => 'rememberme',
		    'id_submit' => 'wp-submit',
		    'remember' => true,
		    'value_username' => NULL,
		    'value_remember' => false );

			echo $personalize_login_pages_plugin->render_login_form(array());
			#wp_login_form($args);
			echo "</div>";
			return false;
		}
		return $false;
	}

	function wpsites_remove_contributor_capabilities() {


		$contributor = get_role( $this->member_admin );
		$admin_role_set = get_role( 'administrator' )->capabilities;

		$caps = $admin_role_set;
		#print_r($admin_role_set);

		$user = wp_get_current_user();


		if (empty($contributor)) {
			// the role doesn't seem to exist so add it for next time
			add_role( $this->$member_admin, $this->group_title, array($this->member_admin_role => 1) );
			return false;
		}

		// this list is definitely not for general team members
		$unwanted_caps = array(
			'switch_themes', 
			'edit_themes',
			'delete_themes',
			'update_themes',
			'install_themes',
			'edit_theme_options',
			'update_core',
			'activate_plugins',
			'edit_plugins',
			'update_plugins',
			'install_plugins',
			'delete_plugins',
		#	'delete_posts',
		#	'manage_options',
		);

		foreach ($admin_role_set as $key=>$val) {
			#echo "<br>|$key| " .  array_key_exists($key, $contributor->capabilities) . ' x ' .in_array($key, $unwanted_caps). ' == ' . (array_key_exists($key, $contributor->capabilities) && in_array($key, $unwanted_caps));
			if (!array_key_exists($key, $contributor->capabilities) && !in_array($key, $unwanted_caps) && !empty($val) ) {
				$contributor->add_cap( $key );
			} elseif ((array_key_exists($key, $contributor->capabilities) && in_array($key, $unwanted_caps)) || empty($val) ) {
				#echo "<br>remove " . $key;
				$contributor->remove_cap( $key );
			} else {
				#echo " no change ";
			}
		}
		#print_r($user->roles);
		#print_r($contributor->capabilities);
		#print_r($admin_role_set);

		//[$this->member_admin] => 1 [switch_themes] => 1 [edit_themes] => 1 [activate_plugins] => 1 [edit_plugins] => 1 [edit_users] => 1 [edit_files] => 1 [manage_options] => 1 [moderate_comments] => 1 [manage_categories] => 1 [manage_links] => 1 [upload_files] => 1 [import] => 1 [unfiltered_html] => 1 [edit_posts] => 1 [edit_others_posts] => 1 [edit_published_posts] => 1 [publish_posts] => 1 [edit_pages] => 1 [read] => 1 [level_10] => 1 [level_9] => 1 [level_8] => 1 [level_7] => 1 [level_6] => 1 [level_5] => 1 [level_4] => 1 [level_3] => 1 [level_2] => 1 [level_1] => 1 [level_0] => 1 [edit_others_pages] => 1 [edit_published_pages] => 1 [publish_pages] => 1 [delete_pages] => 1 [delete_others_pages] => 1 [delete_published_pages] => 1 [delete_posts] => 1 [delete_others_posts] => 1 [delete_published_posts] => 1 [delete_private_posts] => 1 [edit_private_posts] => 1 [read_private_posts] => 1 [delete_private_pages] => 1 [edit_private_pages] => 1 [read_private_pages] => 1 [delete_users] => 1 [create_users] => 1 [unfiltered_upload] => 1 [edit_dashboard] => 1 [update_plugins] => 1 [delete_plugins] => 1 [install_plugins] => 1 [update_themes] => 1 [install_themes] => 1 [update_core] => 1 [list_users] => 1 [remove_users] => 1 [promote_users] => 1 [edit_theme_options] => 1 [delete_themes] => 1 [export] => 1 ) 

		return false;

	} # end of wpsites_remove_contributor_capabilities


	function check_set_user_metadata( $profileuser ) {

		print_r($profileuser) ;

		if (!empty($profileuser->ID)) {
	
			$roles = ( array ) $profileuser->roles;
			if (in_array( $this->member_admin, $roles)) {
				// only show custom fields for team members
				$surveyor_tag = get_user_meta( $profileuser->ID, 'doesSurveys', false) ;

				if (empty($surveyor_tag)) {
					// either tag does not exist or it is empty... we need it to appear in the page list so add
					// blank field to ensure placeholder exists
					update_user_meta( $profileuser->ID, 'doesSurveys', false);
				}

				$surveyor_tag = get_user_meta( $profileuser->ID, 'doesdelete', false) ;

				if (empty($surveyor_tag)) {
					// either tag does not exist or it is empty... we need it to appear in the page list so add
					// blank field to ensure placeholder exists
					update_user_meta( $profileuser->ID, 'doesdelete', false);
				}

				$surveyor_tag = get_user_meta( $profileuser->ID, 'initials', false) ;

				if (empty($surveyor_tag)) {
					// either tag does not exist or it is empty... we need it to appear in the page list so add
					// blank field to ensure placeholder exists
					update_user_meta( $profileuser->ID, 'initials', false);
				}

?>

<h2><?php echo $this->group_title; ?> Roles</h2>
    <table class="form-table">
       <tr>
            <th><label for="user_birthday">Delete Records?</label></th>
            <td>
                <input
                    type="checkbox"
					value="yes"
                    <?php echo get_user_meta($profileuser->ID, 'doesdelete', true)!=false ? 'checked' : '' ; ?>
                    name="doesdelete"
                    id="doesdelete"
                >
                <span class="description">This user can delete records ?</span>
            </td>
        </tr>
        <tr>
            <th><label for="user_birthday">Initials?</label></th>
            <td>
                <input
                    type="text"
					value="<?php echo get_user_meta($profileuser->ID, 'initials', true) ; ?>"
                    name="initials"
                    id="initials"
                >
                <span class="description">Initials (usually unique) for members.</span>
            </td>
        </tr>
    </table>
<?php

			}
		}
	} # end of check_set_user_metadata



} # end of nufUserCapabilitiesClass


$nufucc = new nufUserCapabilitiesClass;





